<?php

namespace KoiLab\WC_Force_Sells\API\Store;

defined( 'ABSPATH' ) or exit;

use Automattic\WooCommerce\StoreApi\Schemas\V1\CartItemSchema;
use KoiLab\WC_Force_Sells\Utilities\Product_Utils;
use WC_Product;

/**
 * Cart extension endpoint for the Store API.
 *
 * @since 1.5.0
 */
class Cart_Extension_Endpoint {

	/** @var string the endpoint identifier */
	public const IDENTIFIER = 'force-sells';

	/**
	 * Store API constructor.
	 *
	 * @since 1.5.0
	 */
	public function __construct() {

		add_action( 'woocommerce_blocks_loaded', [ $this, 'register_store_api_endpoint' ] );
	}

	/**
	 * Registers an extension endpoint for Force Sells with the WooCommerce Store API cart payload.
	 *
	 * Adds a `cart/extensions/force-sells` endpoint to the Store API cart payload.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @return void
	 */
	public function register_store_api_endpoint() : void {

		woocommerce_store_api_register_endpoint_data( [
			'endpoint'        => CartItemSchema::IDENTIFIER,
			'namespace'       => static::IDENTIFIER,
			'data_callback'   => [ $this, 'handle_endpoint_get_request' ],
			'schema_callback' => [ $this, 'get_endpoint_schema' ],
			'schema_type'     => ARRAY_A,
		] );
	}

	/**
	 * Handles a WooCommerce Store API request to the force sells endpoint.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @param array<string, mixed>|mixed $cart_item the cart item
	 * @return array<string, mixed>
	 */
	public function handle_endpoint_get_request( $cart_item ) : array {

		$synced_ids = [];

		if ( $cart_item && isset( $cart_item['forced_by'] ) ) {

			$cart                = WC()->cart;
			$forced_by_cart_item = $cart->get_cart_item( $cart_item['forced_by'] ) ?? false;
			$forced_by_product   = $forced_by_cart_item['data'] ?? false;

			if ( $forced_by_product instanceof WC_Product ) {
				$synced_ids = Product_Utils::get_force_sells( (int) $forced_by_product->get_id(), 'synced' );
			}
		}

		return [
			'synced_ids' => $synced_ids,
		];
	}

	/**
	 * Gets the force sells Store API endpoint schema.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @return array<string, mixed>
	 */
	public function get_endpoint_schema() : array {

		return [
			'synced_ids' => [
				'description' => __( 'The list of synced force sell product ids for the cart item.', 'woocommerce-force-sells' ),
				'type'        => 'array',
				'readonly'    => true,
				'items'       => [
					'type' => 'integer',
				],
			],
		];
	}

}
