<?php

namespace KoiLab\WC_Force_Sells;

defined( 'ABSPATH' ) or exit;

use Automattic\WooCommerce\Admin\Features\ProductBlockEditor\ProductTemplates\GroupInterface;
use Automattic\WooCommerce\Blocks\Utils\CartCheckoutUtils;
use Automattic\WooCommerce\Utilities\FeaturesUtil;
use KoiLab\WC_Force_Sells\Blocks\Integrations\Cart_Integration;
use KoiLab\WC_Force_Sells\API\Store\Cart_Extension_Endpoint;
use KoiLab\WC_Force_Sells\Blocks\Integrations\Product_Editor_Integration;
use WP_Post;

/**
 * Blocks handler.
 *
 * @since 1.5.0
 */
class Blocks {

	/**
	 * Blocks handler constructor.
	 *
	 * @since 1.5.0
	 */
	public function __construct() {

		new Cart_Extension_Endpoint();

		add_action( 'before_woocommerce_init', [ $this, 'handle_woocommerce_blocks_compatibility' ] );
		add_action( 'woocommerce_blocks_loaded', [ $this, 'register_woocommerce_block_integrations' ] );
		add_action( 'woocommerce_block_template_area_product-form_after_add_block_linked-products', [ $this, 'register_woocommerce_product_editor_integrations' ] );
	}

	/**
	 * Initializes the Blocks handler.
	 *
	 * @since 1.5.0
	 *
	 * @return Blocks
	 */
	public static function init() : self {

		return new self();
	}

	/**
	 * Handles compatibility with WooCommerce Blocks.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @return void
	 */
	public function handle_woocommerce_blocks_compatibility() : void {

		if ( ! class_exists( FeaturesUtil::class ) || ! defined( 'WC_FORCE_SELLS_FILE' ) ) {
			return;
		}

		FeaturesUtil::declare_compatibility( 'cart_checkout_blocks', WC_FORCE_SELLS_FILE, true );
		FeaturesUtil::declare_compatibility( 'product_block_editor', WC_FORCE_SELLS_FILE, true );
	}

	/**
	 * Registers the WooCommerce Blocks integrations.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @return void
	 */
	public function register_woocommerce_block_integrations() : void {

		$cart_block_integration_instance = new Cart_Integration();

		add_action( 'woocommerce_blocks_cart_block_registration', function( $integration_registry ) use ( $cart_block_integration_instance ) {
			$integration_registry->register( $cart_block_integration_instance );
		} );
	}

	/**
	 * Registers the WooCommerce Product Editor integrations.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @param GroupInterface $linked_products_group
	 * @return void
	 */
	public function register_woocommerce_product_editor_integrations( GroupInterface $linked_products_group ) : void {

		( new Product_Editor_Integration() )
			->set_linked_products_group( $linked_products_group )
			->add_sections();
	}

	/**
	 * Determines if the checkout page is using the checkout block.
	 *
	 * @since 1.5.0
	 *
	 * @return bool false when using the legacy checkout shortcode
	 */
	public static function is_checkout_block_in_use() : bool {

		return class_exists( CartCheckoutUtils::class ) && CartCheckoutUtils::is_checkout_block_default();
	}

	/**
	 * Determines if a page contains the checkout block.
	 *
	 * @since 1.5.0
	 *
	 * @param int|string|WP_Post $page
	 * @return bool
	 */
	public static function page_contains_checkout_block( $page ) : bool {

		return has_block( 'woocommerce/checkout', $page );
	}

	/**
	 * Determines if a page contains a checkout shortcode.
	 *
	 * @since 1.5.0
	 *
	 * @param int|string|WP_Post $page
	 * @return bool
	 */
	public static function page_contains_checkout_shortcode( $page ) : bool {

		return static::page_contains_shortcode( '[woocommerce_checkout]', $page );
	}

	/**
	 * Determines if the cart page is using the cart block.
	 *
	 * @since 1.5.0
	 *
	 * @return bool false if using the legacy cart shortcode
	 */
	public static function is_cart_block_in_use() : bool {

		return class_exists( CartCheckoutUtils::class ) && CartCheckoutUtils::is_cart_block_default();
	}

	/**
	 * Determines if a page contains the cart block.
	 *
	 * @since 1.5.0
	 *
	 * @param int|string|WP_Post $page
	 * @return bool
	 */
	public static function page_contains_cart_block( $page ) : bool {

		return has_block( 'woocommerce/cart', $page );
	}

	/**
	 * Determines if a page contains a cart shortcode.
	 *
	 * @since 1.5.0
	 *
	 * @param int|string|WP_Post $page
	 * @return bool
	 */
	public static function page_contains_cart_shortcode( $page ) : bool {

		return static::page_contains_shortcode( '[woocommerce_cart]', $page );
	}

	/**
	 * Determines if a page contains a cart or checkout shortcode.
	 *
	 * @since 1.5.0
	 *
	 * @param string $shortcode
	 * @param int|string|WP_Post $page
	 * @return bool
	 */
	private static function page_contains_shortcode( string $shortcode, $page ) : bool {

		if ( is_numeric( $page ) || is_string( $page ) ) {
			$page = get_post( $page );
		}

		return $page instanceof WP_Post && has_shortcode( $page->post_content, $shortcode );
	}

}
