<?php

namespace KoiLab\WC_Force_Sells\Blocks\Integrations;

defined( 'ABSPATH' ) or exit;

use Automattic\WooCommerce\Blocks\Integrations\IntegrationInterface;
use KoiLab\WC_Force_Sells\Blocks;

/**
 * WooCommerce Cart block integration.
 *
 * @since 1.5.0
 */
class Cart_Integration implements IntegrationInterface {

	/** @var string block integration namespace */
	private string $namespace = 'kestrel';

	/** @var string the block the integration is for */
	private string $integration_block_name = 'cart';

	/** @var string[] block integration script dependencies */
	private array $script_dependencies = [
		'wc-blocks-registry',
		'wc-blocks-checkout',
		'wc-settings',
		'wp-element',
		'wp-html-entities',
		'wp-i18n',
		'wp-data',
	];

	/**
	 * Initializes the integration.
	 *
	 * Implements {@see IntegrationInterface::initialize()}.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @return void
	 */
	public function initialize() {

		$this->register_assets();
	}

	/**
	 * Gets the WooCommerce integration name.
	 *
	 * Implements {@see IntegrationInterface::get_name()} with a namespace.
	 * This is specific to WooCommerce blocks and used internally to register the integration.
	 * A corresponding store API route will be registered with this name, {@see Blocks::register_woocommerce_block_integrations()}.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @return string
	 */
	public function get_name() {

		return "{$this->namespace}/force-sells";
	}

	/**
	 * Gets the integration slug (dashes).
	 *
	 * This is mainly used to register the integration where an identifier is required, e.g. when registering assets.
	 *
	 * @since 1.5.0
	 *
	 * @return string
	 */
	protected function get_slug() : string {

		return 'wc-force-sells-cart-block';
	}

	/**
	 * Gets the integration id (underscores).
	 *
	 *  This is mainly used in hook names implemented by the integration.
	 *
	 * @since 1.5.0
	 *
	 * @return string
	 */
	protected function get_id() : string {

		return 'wc_force_sells_cart_block';
	}

	/**
	 * Registers the block integration assets.
	 *
	 * @since 1.5.0
	 *
	 * @return void
	 */
	protected function register_assets() : void {

		wp_register_script( $this->get_script_handle(), $this->get_script_url(), $this->get_script_dependencies(), WC_FORCE_SELLS_VERSION, [ 'in_footer' => true ] );

		wp_set_script_translations( $this->get_script_handle(), 'woocommerce-force-sells' );

		$this->maybe_enqueue_assets();
	}

	/**
	 * Enqueues the block integration assets if the current context is cart or checkout.
	 *
	 * @since 1.5.0
	 *
	 * @return void
	 */
	protected function maybe_enqueue_assets() : void {

		if ( $this->should_enqueue_assets() ) {

			wp_enqueue_script( $this->get_script_handle() );
		}
	}

	/**
	 * Determines whether the block integration assets should be enqueued.
	 *
	 * @since 1.5.0
	 *
	 * @return bool
	 */
	private function should_enqueue_assets() : bool {

		$should_enqueue = ! is_order_received_page() && ! is_checkout_pay_page();

		return $should_enqueue && ( wc_current_theme_is_fse_theme() || Blocks::is_cart_block_in_use() || Blocks::is_checkout_block_in_use() );
	}

	/**
	 * Gets an array of script handles to enqueue in the frontend context.
	 *
	 * Implements {@see IntegrationInterface::get_script_handles()}.
	 *
	 * @since 1.5.0
	 *
	 * @return string[]
	 */
	public function get_script_handles() {

		return [ $this->get_script_handle() ];
	}

	/**
	 * Gets an array of script handles to enqueue in the editor context.
	 *
	 * Implements {@see IntegrationInterface::get_editor_script_handles()}.
	 *
	 * @since 1.5.0
	 *
	 * @return string[]
	 */
	public function get_editor_script_handles() {

		return [ $this->get_script_handle() ];
	}

	/**
	 * Gets the integration script handle.
	 *
	 * @since 1.5.0
	 *
	 * @return string
	 */
	protected function get_script_handle() : string {

		/**
		 * Filters the block integration script handle.
		 *
		 * @since 1.5.0
		 *
		 * @param string $handle
		 * @param IntegrationInterface $integration
		 */
		return (string) apply_filters( "{$this->get_id()}_script_handle", $this->get_slug(), $this );
	}

	/**
	 * Gets the integration script URL.
	 *
	 * @since 1.5.0
	 *
	 * @return string
	 */
	protected function get_script_url() : string {

		/**
		 * Filters the block integration script handle.
		 *
		 * @since 1.5.0
		 *
		 * @param string $handle
		 * @param IntegrationInterface $integration
		 */
		return (string) apply_filters( "{$this->get_id()}_script_url", trailingslashit( WC_FORCE_SELLS_URL ) . 'assets/js/blocks/' . $this->get_slug() . '.js', $this );
	}

	/**
	 * Gets the integration script dependencies.
	 *
	 * @since 1.5.0
	 *
	 * @return string[]
	 */
	protected function get_script_dependencies() : array {

		/**
		 * Filters the block integration script dependencies.
		 *
		 * @since 1.5.0
		 *
		 * @param string[] $dependencies
		 * @param IntegrationInterface $integration
		 */
		return (array) apply_filters( "{$this->get_id()}_script_dependencies", $this->script_dependencies, $this );
	}

	/**
	 * Gets array of key-value pairs of data made available to the block on the client side.
	 *
	 * @since 1.5.0
	 *
	 * @return array<string, mixed>
	 */
	public function get_script_data() {

		/**
		 * Filters the integration script data.
		 *
		 * @since 1.5.0
		 *
		 * @param array<string, mixed> $data
		 * @param IntegrationInterface $integration
		 */
		return (array) apply_filters( "{$this->get_id()}_script_data", [] );
	}

}
