<?php

namespace KoiLab\WC_Force_Sells\Blocks\Integrations;

defined( 'ABSPATH' ) or exit;

use Automattic\WooCommerce\Admin\Features\ProductBlockEditor\ProductTemplates\GroupInterface;

/**
 * WooCommerce Product Editor block integration.
 *
 * @since 1.5.0
 */
class Product_Editor_Integration {

	/** @var string product linked force sells section name */
	private const PRODUCT_LINKED_FORCE_SELLS_SECTION = 'product-linked-force-sells-section';

	/** @var string product linked synced force sells section name */
	private const PRODUCT_LINKED_SYNCED_FORCE_SELLS_SECTION = 'product-linked-synced-force-sells-section';

	/** @var GroupInterface|null linked products group */
	private ?GroupInterface $linked_products_group = null;

	/**
	 * Sets the linked products group.
	 *
	 * @since 1.5.0
	 *
	 * @param GroupInterface $linked_products_group linked products group
	 */
	public function set_linked_products_group( GroupInterface $linked_products_group ) : self {

		$this->linked_products_group = $linked_products_group;

		return $this;
	}

	/**
	 * Adds force sells sections to the product editor.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @return void
	 */
	public function add_sections() : void {

		$linked_products_group = $this->linked_products_group;

		if ( ! $linked_products_group ) {
			return;
		}

		/**
		 * It throws an uncaught exception if you try to add an existing block, group, or section.
		 * It's also possible for third-parties to register their own filters to the same action, so we check the block doesn't exist before adding it.
		 */
		if ( null === $linked_products_group->get_block( static::PRODUCT_LINKED_FORCE_SELLS_SECTION ) ) {

			$linked_products_group->add_section( [
				'id'             => static::PRODUCT_LINKED_FORCE_SELLS_SECTION,
				'order'          => 30,
				'attributes'     => [
					'title'       => __( 'Force sells', 'woocommerce-force-sells' ),
					'description' => sprintf( /* translators: %1$s: "will not be synced" link opening tag. %2$s: "will not be synced" link closing tag. */
						__( 'These products will be added to the cart when the main product is added. Quantity %1$swill not be synced%2$s in case the main product quantity changes.', 'woocommerce-force-sells' ), '<a href="https://woocommerce.com/document/force-sells/#Force-Sells:" target="_blank" rel="noreferrer">', '</a>' ),
				],
				'hideConditions' => [
					[
						'expression' => 'editedProduct.type === "external" || editedProduct.type === "grouped"',
					],
				],
			] )->add_block( [
				'id'         => 'product-linked-force-sells',
				'blockName'  => 'woocommerce/product-linked-list-field',
				'order'      => 10,
				'attributes' => [
					'property'   => 'meta_data._force_sell_ids',
					'emptyState' => [
						'image'         => 'CashRegister',
						'tip'           => __( 'Tip: Add a force sell to the cart with this product, matching quantity! Customers can edit or remove it, but it’s helpful for things like shipping insurance.', 'woocommerce-force-sells' ),
						'isDismissible' => true,
					],
				],
			] );
		}

		if ( null === $linked_products_group->get_block( static::PRODUCT_LINKED_SYNCED_FORCE_SELLS_SECTION ) ) {

			$linked_products_group->add_section( [
				'id'             => static::PRODUCT_LINKED_SYNCED_FORCE_SELLS_SECTION,
				'order'          => 40,
				'attributes'     => [
					'title'       => __( 'Synced force sells', 'woocommerce-force-sells' ),
					'description' => sprintf( /* translators: %1$s: "will be synced" link opening tag. %2$s: "will be synced" link closing tag. */
						__( 'These products will be added to the cart when the main product is added and quantity %1$swill be synced%2$s in case the main product quantity changes.', 'woocommerce-force-sells' ), '<a href="https://woocommerce.com/document/force-sells/#Synced-Force-Sells:" target="_blank" rel="noreferrer">', '</a>' ),
				],
				'hideConditions' => [
					[
						'expression' => 'editedProduct.type === "external" || editedProduct.type === "grouped"',
					],
				],
			] )->add_block( [
				'id'         => 'product-linked-synced-force-sells',
				'blockName'  => 'woocommerce/product-linked-list-field',
				'order'      => 10,
				'attributes' => [
					'property'   => 'meta_data._force_sell_synced_ids',
					'emptyState' => [
						'image'         => 'CashRegister',
						'tip'           => __( 'Tip: Force an extra item in the cart, matching this product’s quantity. Customer’s can’t remove it or change it, but it’s helpful for things like signature required shipping costs', 'woocommerce-force-sells' ),
						'isDismissible' => true,
					],
				],
			] );
		}
	}

}
