<?php

namespace KoiLab\WC_Force_Sells;

defined( 'ABSPATH' ) or exit;

use KoiLab\WC_Force_Sells\Utilities\Product_Utils;
use WC_Order_Item_Product;
use WC_Product;

/**
 * Cart class.
 *
 * @since 1.3.0
 */
class Cart {

	/**
	 * Cart init.
	 *
	 * @since 1.3.0
	 */
	public static function init() {

		add_action( 'woocommerce_ordered_again', [ __CLASS__, 'ordered_again' ], 10, 3 );
		add_filter( 'woocommerce_store_api_product_quantity_minimum', [ __CLASS__, 'maybe_adjust_minimum_quantity' ], 10, 3 );
		add_filter( 'woocommerce_store_api_product_quantity_maximum', [ __CLASS__, 'maybe_adjust_maximum_quantity' ], 10, 3 );
	}

	/**
	 * Makes synced products irremovable.
	 *
	 * @since 1.3.0
	 * @internal
	 *
	 * @param int|mixed $order_id order ID
	 * @param mixed|WC_Order_Item_Product[] $order_items order items
	 * @param array<string,mixed>|mixed $cart the cart content
	 * @return void
	 */
	public static function ordered_again( $order_id, $order_items, &$cart ) : void {

		if ( ! is_array( $cart ) ) {
			return;
		}

		foreach ( $cart as $key => $item ) {
			// Check if this product is forced in itself, so it can't force in others (to prevent adding in loops).
			if ( isset( $item['forced_by'], $cart[ $item['forced_by'] ] ) ) {
				continue;
			}

			// Check if this product is already forcing a cart item. If so, we don't need to handle add to cart logic because qty will be updated by update_force_sell_quantity_in_cart.
			foreach ( $cart as $value ) {
				if ( isset( $value['forced_by'] ) && $key === $value['forced_by'] ) {
					break;
				}
			}

			$synced_ids = Product_Utils::get_valid_force_sells( (int) $item['product_id'], 'synced' );

			if ( ! $synced_ids ) {
				continue;
			}

			foreach ( $cart as $s_key => $s_item ) {
				if ( in_array( $s_item['product_id'], $synced_ids, true ) ) {
					$cart[ $s_key ]['forced_by'] = $key;
				}
			}
		}
	}

	/**
	 * Filters the max quantity property for a cart item in Store API.
	 *
	 * This ensures the customer cannot modify the quantity of _synced_ force sell items.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @param float|int|mixed $max the maximum quantity
	 * @param mixed|WC_Product $product the product object
	 * @param array<string,mixed>|mixed|null $cart_item the cart item if the product exists in the cart, or null
	 * @return float|int|mixed
	 */
	public static function maybe_adjust_maximum_quantity( $max, $product, $cart_item ) {

		return static::maybe_adjust_force_sell_product_cart_quantity( $max, $product, $cart_item );
	}

	/**
	 * Filters the min quantity property for a cart item in Store API.
	 *
	 * This ensures the customer cannot modify the quantity of _synced_ force sell items.
	 *
	 * @since 1.5.0
	 * @internal
	 *
	 * @param float|int|mixed $min the minimum quantity
	 * @param mixed|WC_Product $product the product object
	 * @param array<string,mixed>|mixed|null $cart_item the cart item if the product exists in the cart, or null
	 * @return float|int|mixed
	 */
	public static function maybe_adjust_minimum_quantity( $min, $product, $cart_item ) {

		return static::maybe_adjust_force_sell_product_cart_quantity( $min, $product, $cart_item );
	}

	/**
	 * Maybe adjusts the quantity of a force-sell synced product in the cart.
	 *
	 * @since 1.5.0
	 *
	 * @param float|int|mixed $qty
	 * @param mixed|WC_Product $product
	 * @param array<string, mixed>|mixed $cart_item
	 * @return float|int|mixed
	 */
	private static function maybe_adjust_force_sell_product_cart_quantity( $qty, $product, $cart_item ) {

		if ( $cart_item && is_array( $cart_item ) && isset( $cart_item['forced_by'] ) ) {

			$cart                = WC()->cart;
			$forced_by_cart_item = $cart->get_cart_item( $cart_item['forced_by'] ) ?? false;
			$forced_by_product   = $forced_by_cart_item['data'] ?? false;

			if ( $forced_by_product ) {
				$synced_ids = Product_Utils::get_valid_force_sells( $forced_by_product->get_id(), 'synced' );

				if ( $product instanceof WC_Product && in_array( $product->get_id(), $synced_ids, true ) ) {
					return $forced_by_cart_item['quantity'];
				}
			}
		}

		return $qty;
	}

}
