<?php
/**
 * Product utilities.
 *
 * @since 1.3.0
 */

namespace KoiLab\WC_Force_Sells\Utilities;

defined( 'ABSPATH' ) || exit;

use WP_Post;
use WC_Product;

/**
 * Class Product_Utils.
 */
class Product_Utils {

	/**
	 * Gets the product instance.
	 *
	 * @since 1.3.0
	 *
	 * @param int|WP_Post $the_product post object or post ID of the product
	 * @return WC_Product|null
	 */
	public static function get_product( $the_product ) : ?WC_Product {

		if ( $the_product instanceof WC_Product ) {
			return $the_product;
		}

		return wc_get_product( $the_product ) ?: null;
	}

	/**
	 * Gets the forced sells of a product.
	 *
	 * @since 1.3.0
	 *
	 * @param int $product_id product ID force sells are being retrieved for
	 * @param string $type optional arg to filter the forced sells by type (default empty - accepts: normal, synced)
	 * @param string $return_type whether to return 'ids' (integers) or products as 'objects' (default ids)
	 * @return int[]|WC_Product[]
	 */
	public static function get_force_sells( int $product_id, string $type = '', string $return_type = 'ids' ) : array {

		$allowed_types = [ 'normal', 'synced' ];

		if ( $type && ! in_array( $type, $allowed_types, true ) ) {
			return [];
		}

		$force_sell_ids = [];

		foreach ( $allowed_types as $allowed_type ) {
			if ( $type && $type !== $allowed_type ) {
				continue;
			}

			$meta_key = ( 'normal' === $allowed_type ? '_force_sell_ids' : "_force_sell_{$allowed_type}_ids" );

			$ids = get_post_meta( $product_id, $meta_key, true );

			if ( is_array( $ids ) && ! empty( $ids ) ) {
				$force_sell_ids = array_merge( $force_sell_ids, $ids );
			}
		}

		$force_sell_ids = array_unique( $force_sell_ids );

		return array_values( 'ids' === $return_type ? $force_sell_ids : array_filter( array_map( 'wc_get_product', $force_sell_ids ) ) );
	}

	/**
	 * Gets the valid forced sells of a product.
	 *
	 * @since 1.3.0
	 *
	 * @see Product_Utils::get_force_sells()
	 * @see Product_Utils::force_sell_is_valid()
	 *
	 * @param int $product_id product ID force sells are being retrieved for
	 * @param string $type optional arg to filter the forced sells by type (default empty - accepts: normal, synced)
	 * @param string $return_type whether to return 'ids' (integers) or products as 'objects' (default ids)
	 * @return int[]|WC_Product[]
	 */
	public static function get_valid_force_sells( int $product_id, string $type = '', string $return_type = 'ids' ) : array {

		$force_sells = self::get_force_sells( $product_id, $type, $return_type );

		return array_values( array_filter( $force_sells, [ __CLASS__, 'force_sell_is_valid' ] ) );
	}

	/**
	 * Checks if the provided product is a valid force sell.
	 *
	 * @since 1.3.0
	 *
	 * @param WC_Product|int $the_product product object or ID
	 * @return bool
	 */
	public static function force_sell_is_valid( $the_product ) : bool {

		$product = self::get_product( $the_product );

		return $product && $product->exists() && 'trash' !== $product->get_status();
	}
}
