<?php
/**
 * Plugin Name: Force Sells for WooCommerce
 * Plugin URI: https://woo.com/products/force-sells/
 * Description: Allows you to select products which will be used as force-sells - items which get added to the cart along with other items.
 * Version: 1.5.2
 * Author: Kestrel
 * Author URI: https://kestrelwp.com
 * Requires PHP: 7.4
 * Requires at least: 5.6
 * Tested up to: 6.7.1
 * Requires Plugins: woocommerce
 * Domain: woocommerce-force-sells
 * Domain Path: /languages.
 *
 * WC requires at least: 4.0
 * WC tested up to: 9.6.0
 * Woo: 18678:3ebddfc491ca168a4ea4800b893302b0
 *
 * Copyright: (c) 2012-2024 Kestrel [hey@kestrelwp.com]
 *
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */
defined( 'ABSPATH' ) || exit;

// Load the class autoloader.
require __DIR__ . '/src/Autoloader.php';

if ( ! KoiLab\WC_Force_Sells\Autoloader::init() ) {
	return;
}

// Define plugin file constant.
if ( ! defined( 'WC_FORCE_SELLS_FILE' ) ) {
	define( 'WC_FORCE_SELLS_FILE', __FILE__ );
}

add_action( 'plugins_loaded', [ 'WC_Force_Sells', 'instance' ] );

use KoiLab\WC_Force_Sells\Utilities\Product_Utils;

if ( ! class_exists( 'WC_Force_Sells' ) ) :

	/**
	 * Main plugin class.
	 */
	class WC_Force_Sells extends KoiLab\WC_Force_Sells\Plugin {

		/**
		 * Constructor.
		 */
		protected function __construct() {
			parent::__construct();

			add_action( 'woocommerce_product_options_related', [ $this, 'write_panel_tab' ] );
			add_action( 'woocommerce_process_product_meta', [ $this, 'process_extra_product_meta' ], 1 );
			add_action( 'woocommerce_after_add_to_cart_button', [ $this, 'show_force_sell_products' ] );
			add_action( 'woocommerce_add_to_cart', [ $this, 'add_force_sell_items_to_cart' ], 11, 3 );
			add_action( 'woocommerce_after_cart_item_quantity_update', [ $this, 'update_force_sell_quantity_in_cart' ], 1, 2 );
			add_action( 'woocommerce_remove_cart_item', [ $this, 'update_force_sell_quantity_in_cart' ], 1, 1 );

			// Keep force sell data in the cart.
			add_filter( 'woocommerce_get_cart_item_from_session', [ $this, 'get_cart_item_from_session' ], 10, 2 );
			add_filter( 'woocommerce_get_item_data', [ $this, 'get_linked_to_product_data' ], 10, 2 );
			add_action( 'woocommerce_cart_loaded_from_session', [ $this, 'remove_orphan_force_sells' ] );
			add_action( 'woocommerce_cart_loaded_from_session', [ $this, 'maybe_remove_duplicate_force_sells' ] );

			// Don't allow synced force sells to be removed or change quantity.
			add_filter( 'woocommerce_cart_item_remove_link', [ $this, 'cart_item_remove_link' ], 10, 2 );
			add_filter( 'woocommerce_cart_item_quantity', [ $this, 'cart_item_quantity' ], 10, 2 );

			// Sync with remove/restore link in cart.
			add_action( 'woocommerce_cart_item_removed', [ $this, 'cart_item_removed' ], 30 );
			add_action( 'woocommerce_cart_item_restored', [ $this, 'cart_item_restored' ], 30 );
		}

		/**
		 * If the single instance hasn't been set, set it now.
		 *
		 * @deprecated 1.3.0
		 *
		 * @return WC_Force_Sells
		 */
		public static function get_instance() {
			wc_deprecated_function( __FUNCTION__, '1.3.0', 'WC_Force_Sells::instance()' );

			return self::instance();
		}

		/**
		 * Looks to see if a product with the key of 'forced_by' actually exists and
		 * deletes it if not.
		 */
		public function remove_orphan_force_sells() {
			$cart_contents = WC()->cart->get_cart();

			foreach ( $cart_contents as $key => $value ) {
				if ( isset( $value['forced_by'] ) ) {
					if ( ! array_key_exists( $value['forced_by'], $cart_contents ) ) {
						WC()->cart->remove_cart_item( $key );
					}
				}
			}
		}

		/**
		 * Checks the cart contents to make sure we don't
		 * have duplicated force sell products.
		 *
		 * @since 1.1.19
		 */
		public function maybe_remove_duplicate_force_sells() {
			$cart_contents = WC()->cart->get_cart();
			$product_ids   = [];
			$variation_ids = [];

			foreach ( $cart_contents as $key => $value ) {
				if ( isset( $value['forced_by'] ) ) {
					$product_ids[]   = $value['product_id'];
					$variation_ids[] = $value['variation_id'];
				}
			}

			foreach ( WC()->cart->get_cart() as $key => $value ) {
				if ( ! isset( $value['forced_by'] ) ) {
					// forced product, check if it's a variation
					if ( ( ! empty( $value['variation_id'] ) && in_array( $value['variation_id'], $variation_ids, true ) ) || ( empty( $value['variation_id'] ) && in_array( $value['product_id'], $product_ids, true ) ) ) {
						// it's a duplicate variation, or duplicate product, remove it
						WC()->cart->remove_cart_item( $key );
					}
				}
			}
		}

		/**
		 * Get forced product added again to cart when item is loaded from session.
		 *
		 * @param array $cart_item item in cart
		 * @param array $values item values
		 * @return array cart item
		 */
		public function get_cart_item_from_session( $cart_item, $values ) {
			if ( isset( $values['forced_by'] ) ) {
				$cart_item['forced_by'] = $values['forced_by'];
			}

			return $cart_item;
		}

		/**
		 * Making sure linked products from an item is displayed in cart.
		 *
		 * @param array $data data
		 * @param array $cart_item cart item
		 * @return array
		 */
		public function get_linked_to_product_data( $data, $cart_item ) {
			if ( isset( $cart_item['forced_by'] ) ) {
				$product_key = WC()->cart->find_product_in_cart( $cart_item['forced_by'] );

				if ( ! empty( $product_key ) ) {
					$product_name = WC()->cart->cart_contents[ $product_key ]['data']->get_title();
					$data[]       = [
						'name'    => __( 'Linked to', 'woocommerce-force-sells' ),
						'display' => $product_name,
					];
				}
			}

			return $data;
		}

		/**
		 * Remove link in cart item for Synced Force Sells products.
		 *
		 * @param string $link remove link
		 * @param string $cart_item_key cart item key
		 * @return string link
		 */
		public function cart_item_remove_link( $link, $cart_item_key ) {
			if ( isset( WC()->cart->cart_contents[ $cart_item_key ]['forced_by'] ) ) {
				return '';
			}

			return $link;
		}

		/**
		 * Makes quantity cart item for Synced Force Sells products uneditable.
		 *
		 * @param string $quantity quantity input
		 * @param string $cart_item_key cart item key
		 * @return string quantity input or static text of quantity
		 */
		public function cart_item_quantity( $quantity, $cart_item_key ) {
			if ( isset( WC()->cart->cart_contents[ $cart_item_key ]['forced_by'] ) ) {
				return WC()->cart->cart_contents[ $cart_item_key ]['quantity'];
			}

			return $quantity;
		}

		/**
		 * Renders Force Sells and Synced Force Sells fields in Linked Products tab.
		 *
		 * @since 1.0.0
		 * @internal
		 *
		 * @return void
		 */
		public function write_panel_tab() : void {
			global $post;

			?>
			<p class="form-field">
				<label for="force_sell_ids"><?php esc_html_e( 'Force Sells', 'woocommerce-force-sells' ); ?></label>
				<select
					id="force_sell_ids"
					class="wc-product-search"
					multiple="multiple"
					style="width: 50%;"
					name="force_sell_ids[]"
					data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'woocommerce-force-sells' ); ?>"
					data-action="woocommerce_json_search_products_and_variations"
					data-exclude="<?php echo esc_attr( $post->ID ?: 0 ); ?>"
					data-exclude_type="variable">
					<?php

					/** @var WC_Product[] $products */
					$products = Product_Utils::get_force_sells( (int) $post->ID, 'normal', 'objects' );

					foreach ( $products as $product ) :

						printf( '<option value="%1$s" selected="selected">%2$s</option>', esc_attr( $product->get_id() ), wp_kses_post( $product->get_formatted_name() ) );

					endforeach;
					?>
				</select>
				<?php echo wc_help_tip( esc_html__( 'These products will be added to the cart when the main product is added. Quantity will not be synced in case the main product quantity changes.', 'woocommerce-force-sells' ) ); ?>
			</p>
			<p class="form-field">
				<label for="force_sell_synced_ids"><?php esc_html_e( 'Synced Force Sells', 'woocommerce-force-sells' ); ?></label>
				<select
					id="force_sell_synced_ids"
					class="wc-product-search"
					multiple="multiple"
					style="width: 50%;"
					name="force_sell_synced_ids[]"
					data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'woocommerce-force-sells' ); ?>"
					data-action="woocommerce_json_search_products_and_variations"
					data-exclude="<?php echo esc_attr( $post->ID ?: 0 ); ?>"
					data-exclude_type="variable">
					<?php

					$products = Product_Utils::get_force_sells( (int) $post->ID, 'synced', 'objects' );

					foreach ( $products as $product ) :

						printf( '<option value="%1$s" selected="selected">%2$s</option>', esc_attr( $product->get_id() ), wp_kses_post( $product->get_formatted_name() ) );

					endforeach;
					?>
				</select>
				<?php echo wc_help_tip( esc_html__( 'These products will be added to the cart when the main product is added and quantity will be synced in case the main product quantity changes.', 'woocommerce-force-sells' ) ); ?>
			</p>
			<?php
		}

		/**
		 * Save Force Sell Ids into post meta when product is saved.
		 *
		 * @param int $post_id post ID
		 */
		public function process_extra_product_meta( $post_id ) {
			// phpcs:disable WordPress.Security.NonceVerification
			foreach ( [ 'force_sell_ids', 'force_sell_synced_ids' ] as $field_name ) {
				$meta_name = "_{$field_name}";

				if ( isset( $_POST[ $field_name ] ) ) {
					$force_sells = array_filter( array_map( 'absint', wp_unslash( $_POST[ $field_name ] ) ) );

					update_post_meta( $post_id, $meta_name, $force_sells );
				} else {
					delete_post_meta( $post_id, $meta_name );
				}
			}
			// phpcs:enable WordPress.Security.NonceVerification
		}

		/**
		 * Displays information of what linked products that will get added when current
		 * product is added to cart.
		 */
		public function show_force_sell_products() {
			global $post;

			$force_sells = Product_Utils::get_valid_force_sells( (int) $post->ID, '', 'objects' );

			if ( ! $force_sells ) {
				return;
			}

			/**
			 * Filters whether to show the force sell product price.
			 *
			 * @since 1.4.0
			 *
			 * @param bool $show_price whether to show the price
			 */
			$show_price = apply_filters( 'wc_force_sell_show_price', true );

			$params = [
				'force_sells' => $force_sells,
				'show_price'  => $show_price,
			];

			wc_get_template( 'single-product/force-sells.php', $params, '', WC_FORCE_SELLS_PATH . 'templates/' );
		}

		/**
		 * Add linked products when current product is added to the cart.
		 *
		 * @since 1.4.0
		 * @internal
		 *
		 * @param mixed|string $cart_item_key cart item key
		 * @param int|mixed $product_id product ID
		 * @param int|mixed $quantity quantity added to cart
		 * @throws Exception Notice message when the forced item is out of stock and parent isn't added.
		 */
		public function add_force_sell_items_to_cart( $cart_item_key, $product_id, $quantity ) {

			// check if this product is forced in itself, so it can't force in others (to prevent adding in loops)
			if ( isset( WC()->cart->cart_contents[ $cart_item_key ]['forced_by'] ) ) {

				$forced_by_key = WC()->cart->cart_contents[ $cart_item_key ]['forced_by'];

				if ( isset( WC()->cart->cart_contents[ $forced_by_key ] ) ) {
					return;
				}
			}

			// check if this product is already forcing a cart item. If so, we don't need to handle add to cart logic because qty will be updated by update_force_sell_quantity_in_cart
			foreach ( WC()->cart->cart_contents as $value ) {
				if ( isset( $value['forced_by'] ) && $cart_item_key === $value['forced_by'] ) {
					return;
				}
			}

			// don't force products on the manual payment page (they are already forced when creating the order)
			if ( is_checkout_pay_page() ) {
				return;
			}

			$product        = wc_get_product( $product_id );
			$force_sell_ids = Product_Utils::get_valid_force_sells( (int) $product_id );
			$synced_ids     = Product_Utils::get_valid_force_sells( (int) $product_id, 'synced' );

			foreach ( $force_sell_ids as $force_sell_id ) {

				$filter_args       = [];
				$forced_product    = wc_get_product( ( $force_sell_id ) );
				$sold_individually = false;

				if ( $synced_ids ) {
					if ( in_array( $force_sell_id, $synced_ids, true ) ) {
						$filter_args['forced_by'] = $cart_item_key;
					}
				}

				if ( $forced_product && $forced_product->get_sold_individually() ) {

					foreach ( WC()->cart->cart_contents as $cart_contents ) {

						if ( $cart_contents['product_id'] === $force_sell_id ) {

							$sold_individually = true;
							break;
						}
					}
				}

				// products sold individually cannot be force added more than once in cart, if already present
				if ( $forced_product && $sold_individually ) {
					continue;
				}

				/**
				 * Filters the properties used when adding the force sell to cart.
				 *
				 * @since 1.4.0
				 *
				 * @param array<string, mixed> $params params passed to the {@see WC_Cart::add_to_cart()} function
				 */
				$params = apply_filters( 'wc_force_sell_add_to_cart_product', [
					'id'           => $forced_product->is_type( 'variation' ) ? $forced_product->get_parent_id() : $force_sell_id,
					'quantity'     => $quantity,
					'variation_id' => $forced_product->is_type( 'variation' ) ? $force_sell_id : '',
					'variation'    => '',
				], WC()->cart->cart_contents[ $cart_item_key ] );

				// force-add product to cart
				$result = WC()->cart->add_to_cart( $params['id'], $params['quantity'], $params['variation_id'], $params['variation'], $filter_args );

				/**
				 * Filters whether to remove the main product if the forced sell product failed to be added to cart.
				 *
				 * @since 1.4.0
				 *
				 * @param bool $disallow_no_stock whether to remove the main product
				 */
				if ( empty( $result ) && apply_filters( 'wc_force_sell_disallow_no_stock', true ) ) {

					WC()->cart->remove_cart_item( $cart_item_key );

					/* translators: Placeholder: %s - Product title */
					throw new Exception( sprintf( esc_html__( '%s will also be removed as they\'re sold together.', 'woocommerce-force-sells' ), esc_html( $product->get_title() ) ) );
				}
			}
		}

		/**
		 * When a product in the cart has its quantity updated, update any synced products it is forcing to match.
		 *
		 * @since 1.3.0
		 * @internal
		 *
		 * @param mixed|string $cart_item_key cart item key
		 * @param float|int|mixed $quantity quantity
		 * @return void
		 */
		public function update_force_sell_quantity_in_cart( $cart_item_key, $quantity = 0 ) : void {

			if ( ! empty( WC()->cart->cart_contents[ $cart_item_key ] ) ) {

				if ( 0 === $quantity || 0 > $quantity ) {
					$quantity = 0;
				} else {
					$quantity = WC()->cart->cart_contents[ $cart_item_key ]['quantity'] ?: 0;
				}

				foreach ( WC()->cart->cart_contents as $key => $value ) {

					if ( isset( $value['forced_by'], $value['product_id'] ) && $cart_item_key === $value['forced_by'] ) {

						$forced_product = wc_get_product( $value['product_id'] );

						if ( ! $forced_product || $forced_product->is_sold_individually() ) {
							continue;
						}

						/**
						 * Filters the new quantity of synced force sell products in cart.
						 *
						 * @since 1.4.0
						 *
						 * @param float|int $quantity the new quantity of the synced force sell product
						 */
						$new_quantity = apply_filters( 'wc_force_sell_update_quantity', $quantity, WC()->cart->cart_contents[ $key ] );

						WC()->cart->set_quantity( $key, $new_quantity );
					}
				}
			}
		}

		/**
		 * When an item gets removed from the cart, do the same for forced sells.
		 *
		 * @since 1.3.0
		 * @internal
		 *
		 * @param mixed|string $cart_item_key cart item key
		 * @return void
		 */
		public function cart_item_removed( $cart_item_key ) : void {

			if ( ! WC()->cart ) {
				return;
			}

			foreach ( WC()->cart->get_cart() as $key => $value ) {

				if ( isset( $value['forced_by'] ) && $cart_item_key === $value['forced_by'] ) {

					WC()->cart->remove_cart_item( $key );
				}
			}
		}

		/**
		 * When an item gets restored in the cart, do the same for forced sells.
		 *
		 * @since 1.3.0
		 * @internal
		 *
		 * @param mixed|string $cart_item_key cart item key
		 * @return void
		 */
		public function cart_item_restored( $cart_item_key ) : void {

			if ( ! WC()->cart ) {
				return;
			}

			foreach ( WC()->cart->removed_cart_contents as $key => $value ) {

				if ( isset( $value['forced_by'] ) && $cart_item_key === $value['forced_by'] ) {

					WC()->cart->restore_cart_item( $key );
				}
			}
		}

	}

endif;
